@@ -73,6 +73,11 @@ var Stream = (function StreamClosure() {
       this.pos = end;
       return bytes.subarray(pos, end);
     },
+    peekByte: function Stream_peekByte() {
+      var peekedByte = this.getByte();
+      this.pos--;
+      return peekedByte;
+    },
     peekBytes: function Stream_peekBytes(length) {
       var bytes = this.getBytes(length);
       this.pos -= bytes.length;
@@ -202,6 +207,11 @@ var DecodeStream = (function DecodeStreamClosure() {
       this.pos = end;
       return this.buffer.subarray(pos, end);
     },
+    peekByte: function DecodeStream_peekByte() {
+      var peekedByte = this.getByte();
+      this.pos--;
+      return peekedByte;
+    },
     peekBytes: function DecodeStream_peekBytes(length) {
       var bytes = this.getBytes(length);
       this.pos -= bytes.length;
@@ -374,10 +384,10 @@ var FlateStream = (function FlateStreamClosure() {
 
     var cmf = str.getByte();
     var flg = str.getByte();
-    if (cmf == -1 || flg == -1) {
+    if (cmf === -1 || flg === -1) {
       error('Invalid header in flate stream: ' + cmf + ', ' + flg);
     }
-    if ((cmf & 0x0f) != 0x08) {
+    if ((cmf & 0x0f) !== 0x08) {
       error('Unknown compression method in flate stream: ' + cmf + ', ' + flg);
     }
     if ((((cmf << 8) + flg) % 31) !== 0) {
@@ -444,7 +454,7 @@ var FlateStream = (function FlateStreamClosure() {
   };
 
   FlateStream.prototype.generateHuffmanTable =
-    function flateStreamGenerateHuffmanTable(lengths) {
+      function flateStreamGenerateHuffmanTable(lengths) {
     var n = lengths.length;
 
     // find max code length
@@ -463,7 +473,7 @@ var FlateStream = (function FlateStreamClosure() {
          len <= maxLen;
          ++len, code <<= 1, skip <<= 1) {
       for (var val = 0; val < n; ++val) {
-        if (lengths[val] == len) {
+        if (lengths[val] === len) {
           // bit-reverse the code
           var code2 = 0;
           var t = code;
@@ -513,7 +523,7 @@ var FlateStream = (function FlateStreamClosure() {
         error('Bad block header in flate stream');
       }
       check |= (b << 8);
-      if (check != (~blockLen & 0xffff) &&
+      if (check !== (~blockLen & 0xffff) &&
           (blockLen !== 0 || check !== 0)) {
         // Ignoring error for bad "empty" block (see issue 1277)
         error('Bad uncompressed block length in flate stream');
@@ -527,7 +537,7 @@ var FlateStream = (function FlateStreamClosure() {
       var end = bufferLength + blockLen;
       this.bufferLength = end;
       if (blockLen === 0) {
-        if (str.peekBytes(1).length === 0) {
+        if (str.peekByte() === -1) {
           this.eof = true;
         }
       } else {
@@ -544,10 +554,10 @@ var FlateStream = (function FlateStreamClosure() {
 
     var litCodeTable;
     var distCodeTable;
-    if (hdr == 1) { // compressed block, fixed codes
+    if (hdr === 1) { // compressed block, fixed codes
       litCodeTable = fixedLitCodeTab;
       distCodeTable = fixedDistCodeTab;
-    } else if (hdr == 2) { // compressed block, dynamic codes
+    } else if (hdr === 2) { // compressed block, dynamic codes
       var numLitCodes = this.getBits(5) + 257;
       var numDistCodes = this.getBits(5) + 1;
       var numCodeLenCodes = this.getBits(4) + 4;
@@ -569,11 +579,11 @@ var FlateStream = (function FlateStreamClosure() {
       var bitsLength, bitsOffset, what;
       while (i < codes) {
         var code = this.getCode(codeLenCodeTab);
-        if (code == 16) {
+        if (code === 16) {
           bitsLength = 2; bitsOffset = 3; what = len;
-        } else if (code == 17) {
+        } else if (code === 17) {
           bitsLength = 3; bitsOffset = 3; what = (len = 0);
-        } else if (code == 18) {
+        } else if (code === 18) {
           bitsLength = 7; bitsOffset = 11; what = (len = 0);
         } else {
           codeLengths[i++] = len = code;
@@ -607,7 +617,7 @@ var FlateStream = (function FlateStreamClosure() {
         buffer[pos++] = code1;
         continue;
       }
-      if (code1 == 256) {
+      if (code1 === 256) {
         this.bufferLength = pos;
         return;
       }
@@ -874,7 +884,7 @@ var JpegStream = (function JpegStreamClosure() {
       var jpegImage = new JpegImage();
 
       // checking if values needs to be transformed before conversion
-      if (this.dict && isArray(this.dict.get('Decode'))) {
+      if (this.forceRGB && this.dict && isArray(this.dict.get('Decode'))) {
         var decodeArr = this.dict.get('Decode');
         var bitsPerComponent = this.dict.get('BitsPerComponent') || 8;
         var decodeArrLength = decodeArr.length;
@@ -928,7 +938,7 @@ var JpegStream = (function JpegStreamClosure() {
       function JpegStream_isNativelyDecodable(xref, res) {
     var cs = ColorSpace.parse(this.dict.get('ColorSpace', 'CS'), xref, res);
     var numComps = cs.numComps;
-    return numComps == 1 || numComps == 3;
+    return numComps === 1 || numComps === 3;
   };
 
   return JpegStream;
@@ -1033,7 +1043,8 @@ var Jbig2Stream = (function Jbig2StreamClosure() {
 
     var jbig2Image = new Jbig2Image();
 
-    var chunks = [], decodeParams = this.dict.get('DecodeParms');
+    var chunks = [], xref = this.dict.xref;
+    var decodeParams = xref.fetchIfRef(this.dict.get('DecodeParms'));
 
     // According to the PDF specification, DecodeParms can be either
     // a dictionary, or an array whose elements are dictionaries.
@@ -1042,7 +1053,7 @@ var Jbig2Stream = (function Jbig2StreamClosure() {
         warn('JBIG2 - \'DecodeParms\' array with multiple elements ' +
              'not supported.');
       }
-      decodeParams = decodeParams[0];
+      decodeParams = xref.fetchIfRef(decodeParams[0]);
     }
     if (decodeParams && decodeParams.has('JBIG2Globals')) {
       var globalsStream = decodeParams.get('JBIG2Globals');
@@ -1148,7 +1159,7 @@ var Ascii85Stream = (function Ascii85StreamClosure() {
     var i;
 
     // special code for z
-    if (c == Z_LOWER_CHAR) {
+    if (c === Z_LOWER_CHAR) {
       buffer = this.ensureBuffer(bufferLength + 4);
       for (i = 0; i < 4; ++i) {
         buffer[bufferLength + i] = 0;
@@ -1165,7 +1176,7 @@ var Ascii85Stream = (function Ascii85StreamClosure() {
 
         input[i] = c;
 
-        if (c === EOF || c == TILDA_CHAR) {
+        if (c === EOF || c === TILDA_CHAR) {
           break;
         }
       }
@@ -1272,7 +1283,7 @@ var RunLengthStream = (function RunLengthStreamClosure() {
     // (in addition to the second byte from the header), n = 129 through 255 -
     // duplicate the second byte from the header (257 - n) times, n = 128 - end.
     var repeatHeader = this.str.getBytes(2);
-    if (!repeatHeader || repeatHeader.length < 2 || repeatHeader[0] == 128) {
+    if (!repeatHeader || repeatHeader.length < 2 || repeatHeader[0] === 128) {
       this.eof = true;
       return;
     }
@@ -1765,7 +1776,7 @@ var CCITTFaxStream = (function CCITTFaxStreamClosure() {
     while ((code1 = this.lookBits(12)) === 0) {
       this.eatBits(1);
     }
-    if (code1 == 1) {
+    if (code1 === 1) {
       this.eatBits(12);
     }
     if (this.encoding > 0) {
@@ -2018,36 +2029,43 @@ var CCITTFaxStream = (function CCITTFaxStreamClosure() {
         }
       }
 
-      if (this.byteAlign) {
-        this.inputBits &= ~7;
-      }
-
       var gotEOL = false;
 
-      if (!this.eoblock && this.row == this.rows - 1) {
+      if (!this.eoblock && this.row === this.rows - 1) {
         this.eof = true;
-      } else {
+      } else if (this.eoline || !this.byteAlign) {
         code1 = this.lookBits(12);
-        while (code1 === 0) {
-          this.eatBits(1);
-          code1 = this.lookBits(12);
+        if (this.eoline) {
+          while (code1 !== EOF && code1 !== 1) {
+            this.eatBits(1);
+            code1 = this.lookBits(12);
+          }
+        } else {
+          while (code1 === 0) {
+            this.eatBits(1);
+            code1 = this.lookBits(12);
+          }
         }
-        if (code1 == 1) {
+        if (code1 === 1) {
           this.eatBits(12);
           gotEOL = true;
-        } else if (code1 == EOF) {
+        } else if (code1 === EOF) {
           this.eof = true;
         }
       }
 
+      if (this.byteAlign && !gotEOL) {
+        this.inputBits &= ~7;
+      }
+
       if (!this.eof && this.encoding > 0) {
         this.nextLine2D = !this.lookBits(1);
         this.eatBits(1);
       }
 
-      if (this.eoblock && gotEOL) {
+      if (this.eoblock && gotEOL && this.byteAlign) {
         code1 = this.lookBits(12);
-        if (code1 == 1) {
+        if (code1 === 1) {
           this.eatBits(12);
           if (this.encoding > 0) {
             this.lookBits(1);
@@ -2056,7 +2074,7 @@ var CCITTFaxStream = (function CCITTFaxStreamClosure() {
           if (this.encoding >= 0) {
             for (i = 0; i < 4; ++i) {
               code1 = this.lookBits(12);
-              if (code1 != 1) {
+              if (code1 !== 1) {
                 info('bad rtc code: ' + code1);
               }
               this.eatBits(12);
@@ -2071,11 +2089,11 @@ var CCITTFaxStream = (function CCITTFaxStreamClosure() {
       } else if (this.err && this.eoline) {
         while (true) {
           code1 = this.lookBits(13);
-          if (code1 == EOF) {
+          if (code1 === EOF) {
             this.eof = true;
             return null;
           }
-          if ((code1 >> 1) == 1) {
+          if ((code1 >> 1) === 1) {
             break;
           }
           this.eatBits(1);
@@ -2151,7 +2169,7 @@ var CCITTFaxStream = (function CCITTFaxStreamClosure() {
     var limitValue = limit || 0;
     for (var i = start; i <= end; ++i) {
       var code = this.lookBits(i);
-      if (code == EOF) {
+      if (code === EOF) {
         return [true, 1, false];
       }
       if (i < end) {
@@ -2159,7 +2177,7 @@ var CCITTFaxStream = (function CCITTFaxStreamClosure() {
       }
       if (!limitValue || code >= limitValue) {
         var p = table[code - limitValue];
-        if (p[0] == i) {
+        if (p[0] === i) {
           this.eatBits(i);
           return [true, p[1], true];
         }
@@ -2197,7 +2215,7 @@ var CCITTFaxStream = (function CCITTFaxStreamClosure() {
     var p;
     if (this.eoblock) {
       code = this.lookBits(12);
-      if (code == EOF) {
+      if (code === EOF) {
         return 1;
       }
 
@@ -2233,7 +2251,7 @@ var CCITTFaxStream = (function CCITTFaxStreamClosure() {
     var code, p;
     if (this.eoblock) {
       code = this.lookBits(13);
-      if (code == EOF) {
+      if (code === EOF) {
         return 1;
       }
       if ((code >> 7) === 0) {
@@ -2381,7 +2399,7 @@ var LZWStream = (function LZWStreamClosure() {
         } else {
           currentSequence[currentSequenceLength++] = currentSequence[0];
         }
-      } else if (code == 256) {
+      } else if (code === 256) {
         codeLength = 9;
         nextCode = 258;
         currentSequenceLength = 0;
